import requests
import os
import json


def _port():
    port = os.environ.get("SDK_SPEAK_PORT", None)
    if port is None:
        raise RuntimeError(
            "CTOAI_PYSDK-8080: The CTO.ai Ops SDK requires a daemon process to be running; this does not appear to be the case."
        )
    return port


def _make_requester(endpoint):
    return lambda data: requests.post(
        f"http://127.0.0.1:{_port()}/{endpoint}", json=data
    ).raise_for_status()


print = _make_requester("print")
start_spinner = _make_requester("start-spinner")
stop_spinner = _make_requester("stop-spinner")
start_progress = _make_requester("progress-bar/start")
advance_progress = _make_requester("progress-bar/advance")
stop_progress = _make_requester("progress-bar/stop")
track = _make_requester("track")
start_op = _make_requester("track")
set_state = _make_requester("state/set")
set_config = _make_requester("config/set")


def _make_sync_requester(endpoint):
    def requester(body):
        response = requests.post(f"http://127.0.0.1:{_port()}/{endpoint}", json=body)

        response.raise_for_status()
        response_data = response.json()

        return response_data["value"]

    return requester


get_state = _make_sync_requester("state/get")
get_all_state = _make_sync_requester("state/get-all")
get_config = _make_sync_requester("config/get")
get_all_config = _make_sync_requester("config/get-all")
delete_config = _make_sync_requester("config/delete")
events = _make_sync_requester("events")


def _make_async_requester(endpoint):
    def requester(body):
        response = requests.post(f"http://127.0.0.1:{_port()}/{endpoint}", json=body)

        response.raise_for_status()
        response_data = response.json()

        with open(response_data["replyFilename"]) as infile:
            return json.load(infile)

    return requester


prompt = _make_async_requester("prompt")
get_secret = _make_async_requester("secret/get")
set_secret = _make_async_requester("secret/set")



from . import daemon_request
from typing import Optional
from .sdk import get_interface_type


def bold(text: str) -> str:
    """Format the given text as bold on the current interface"""
    return f"*{text}*" if get_interface_type() == "slack" else f"\033[1m{text}\033[0m"


def italic(text: str) -> str:
    """Format the given text as italic on the current interface"""
    return f"_{text}_" if get_interface_type() == "slack" else f"\033[3m{text}\033[23m"


def print(text: str):
    """Print a string to the user"""
    daemon_request.print({"text": text})


def spinner_start(text: str):
    """Start a spinner with the given label"""
    daemon_request.start_spinner({"text": text})


def spinner_stop(text: Optional[str] = None):
    """Stop the running spinner, if one is running"""
    daemon_request.stop_spinner({} if text is None else {"text": text})


def progress_bar_start(length: int, initial: int = 0, message: Optional[str] = None):
    """Start a progress bar with the given length"""
    daemon_request.start_progress(
        {"length": length, "initial": initial, "text": message}
    )


def progress_bar_advance(increment: int = 1):
    """Advance the running progress bar by increment"""
    daemon_request.advance_progress({"increment": increment})


def progress_bar_stop(message: Optional[str] = None):
    """Stop the current progress bar"""
    daemon_request.stop_progress({} if message is None else {"text": message})



from . import daemon_request
from typing import Iterable, Optional, Union


def input(
    name: str,
    message: str,
    *,
    default: Optional[str] = None,
    allowEmpty: bool = False,
    flag: Optional[str] = None,
) -> str:
    """Prompt the user for a free-form single-line string"""
    return daemon_request.prompt(dict(type="input", **locals()))[name]


def number(
    name: str,
    message: str,
    *,
    default: Optional[int] = None,
    minimum: Optional[int] = None,
    maximum: Optional[int] = None,
    flag: Optional[str] = None,
):
    """Prompt the user for an integer"""
    return daemon_request.prompt(dict(type="number", **locals()))[name]


def secret(name: str, message: str, *, flag: Optional[str] = None) -> str:
    """Prompt the user for a secret"""
    return daemon_request.prompt(dict(type="secret", **locals()))[name]


def password(
    name: str, message: str, *, confirm: bool = False, flag: Optional[str] = None
) -> str:
    """Prompt the user for a password"""
    return daemon_request.prompt(dict(type="password", **locals()))[name]


def confirm(
    name: str, message: str, *, default: bool = False, flag: Optional[str] = None
):
    """Prompt the user for a yes/no answer"""
    return daemon_request.prompt(dict(type="confirm", **locals()))[name]


def list(
    name: str,
    message: str,
    *,
    choices: Iterable[str],
    default: Optional[Union[str, int]] = None,
    flag: Optional[str] = None,
):
    """Prompt the user for a selection from a list of options"""
    return daemon_request.prompt(dict(type="list", **locals()))[name]


def autocomplete(
    name: str,
    message: str,
    *,
    choices: Iterable[str],
    default: Optional[Union[str, int]] = None,
    flag: Optional[str] = None,
):
    """Prompt the user for a selection from a list of options, with autocomplete on the values"""
    return daemon_request.prompt(dict(type="autocomplete", **locals()))[name]


def checkbox(
    name: str,
    message: str,
    *,
    choices: Iterable[str],
    default: Optional[Iterable[Union[str, int]]] = None,
    flag: Optional[str] = None,
):
    """Prompt the user for a selection of any number members of a list of strings"""
    return daemon_request.prompt(dict(type="checkbox", **locals()))[name]


def editor(name: str, message: str, default: Optional[str] = None):
    """Prompt the user for a multiline freeform string"""
    return daemon_request.prompt(dict(type="editor", **locals()))[name]


VARIANT_DATETIME = "datetime"
VARIANT_DATE = "date"
VARIANT_TIME = "time"


def datetime(
    name: str,
    message: str,
    *,
    variant: str = VARIANT_DATETIME,
    default=None,
    minimum=None,
    maximum=None,
    flag: Optional[str] = None,
):
    """Prompt the user for a date and/or time"""
    return daemon_request.prompt(dict(type="datetime", **locals()))[name]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cto-ai",
    version="2.3.1",
    author="CTO.ai",
    author_email="devops@cto.ai",
    description="SDK for The Ops Platform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cto-ai/sdk-python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires=">=3.6",
    install_requires=[
        "requests>=2.22"
    ],
)



import os, json, os.path, typing
from os.path import abspath
from requests.exceptions import RequestException
from datetime import datetime
from typing import Optional, List

from . import daemon_request


def get_host_os() -> str:
    """Gets the OS of the current host"""
    return os.environ.get("OPS_HOST_PLATFORM", "unknown")


def get_interface_type() -> str:
    """Gets the interface type that the op is currently running through"""
    return os.environ.get("SDK_INTERFACE_TYPE", "terminal")


def home_dir() -> str:
    """Return the location of the user home directory"""
    return os.environ.get("SDK_HOME_DIR", "/root")


log = print

# DEPRECATED: please use `sdk.home_dir()` instead
def get_state_path() -> str:
    return abspath(os.environ.get("SDK_STATE_DIR", ""))


# DEPRECATED: incompatible with current config API
def get_config_path() -> str:
    return abspath(os.environ.get("SDK_CONFIG_DIR", ""))


class JsonStore:
    def __init__(self, getter, getter_all, setter, deleter=None):
        self.getter = getter
        self.getter_all = getter_all
        self.setter = setter
        self.deleter = deleter

    def get_all(self):
        return self.getter_all({})

    def get(self, key: str):
        return self.getter({"key": key})

    def set(self, key: str, value: str):
        self.setter({"key": key, "value": value})
        return self.getter_all({})

    def delete(self, key: str):
        if self.deleter is None:
            raise RuntimeError("Delete method not implemented.")
        return self.deleter({"key": key})


# DEPRECATED: state is used by deprecated workflows feature
state = JsonStore(
    daemon_request.get_state, daemon_request.get_all_state, daemon_request.set_state
)

config = JsonStore(
    daemon_request.get_config,
    daemon_request.get_all_config,
    daemon_request.set_config,
    daemon_request.delete_config,
)


def track(
    tags: typing.Iterable[str], event: str, metadata: typing.Mapping[str, typing.Any]
):
    """Track an analytics event with the analytics backend"""
    try:
        daemon_request.track(dict(tags=tags, event=event, **metadata))

    # It remains unclear what to do with these errors
    except RequestException:
        pass


def start(
    workflowName: str
):
    """Start a workflow from the sdk"""
    try:
        daemon_request.start_op(dict(trigger=True, workflowName=workflowName, tags=["trigger"]))

    # It remains unclear what to do with these errors
    except RequestException:
        pass


def events(start: str, end: Optional[str] = None) -> List[dict]:
    if end is None:
        end = datetime.now().isoformat()
    return daemon_request.events({"start": start, "end": end})


def get_secret(key: str, hidden: Optional[bool] = False) -> str:
    """Get a secret from the secret store by key"""
    return daemon_request.get_secret({"key": key, "hidden": hidden})[key]


def set_secret(key: str, value: str) -> str:
    """Set a secret in the secret store by key"""
    return daemon_request.set_secret({"key": key, "value": value})["key"]
